

#ifndef	_PROATMAPI_H_
#define	_PROATMAPI_H_

#include "queue.h"
#include "devioctl.h"


#define 	MAX_VCS	 256

#define     MIN_VPI 0
#define     MAX_VPI 0
#define     MIN_VCI 32
#define     MAX_VCI 4096
#define     MAX_PDU_SIZE  9180

#define MAX_ATM_ADAPTERS        4
#define MAX_NAME_LENGTH         1024



#define CONNECTION_PVC     0
#define CONNECTION_SVC_OUT 1
#define CONNECTION_SVC_IN  2

#define QOS_UBR 0
#define QOS_CBR 1
#define QOS_VBR 2
#define QOS_ABR 4

#define  NO_GROUP              0
#define  ZERO_FLAGS            0
#define  NULL_SOCKET          -1
#define  FAIL                 -1
#define  SUCCESS               1


#define LAYER_AAL5 0
#define LAYER_AAL0 1

typedef struct _SVC_SETTINGS
{
    union
    {
        char ServerAddress[20];
        char Selector;
    };
 

}SVC_SETTINGS, *PSVC_SETTINGS;

typedef struct _PVC_SETTINGS
{
    unsigned int Vpi;
    unsigned int Vci;
	unsigned int Gfc;
	unsigned int Pt; 
	unsigned int Clp;

}PVC_SETTINGS, *PPVC_SETTINGS;


typedef struct _UBR_SETTINGS
{
    unsigned long PeakCellRate;    

} UBR_SETTINGS, *PUBR_SETTINGS;

typedef struct _CBR_SETTINGS
{
    unsigned long PeakCellRate;    

} CBR_SETTINGS, *PCBR_SETTINGS;

typedef struct _VBR_SETTINGS
{
    unsigned long PeakCellRate;
    unsigned long SubstainableCellRate;
    unsigned long MaxBurstSize;        

}VBR_SETTINGS, *PVBR_SETTINGS;

typedef struct _ABR_SETTINGS
{
    unsigned long PeakCellRate;
    unsigned long MinimumCellRate;
    unsigned long AllowedCellRate;
    unsigned long InitialCellRate;
    unsigned long RateIncreaseFactor;
    unsigned long RateDecreaseFactor;
    unsigned long CuttofDecreaseFactor;

}ABR_SETTINGS;


typedef struct _QOS_SETTINGS
{

    union
    {
        UBR_SETTINGS Ubr;
        CBR_SETTINGS Cbr;
        VBR_SETTINGS Vbr;
        ABR_SETTINGS Abr;
    };

}QOS_SETTINGS, *PQOS_SETTINGS;

typedef struct _VC_SETTINGS
{
    union
    {
        SVC_SETTINGS Svc;
        PVC_SETTINGS Pvc;
    };

}VC_SETTINGS, P_VC_SETTINGS;
 
typedef struct _ATM_SETTINGS
{
    int VcType;
    VC_SETTINGS VcSettings;
    int QualityOfServices;
    int SetQosSettings;
    QOS_SETTINGS QosSettings;
    int AalType;
    int XmitFlow;
    int ReceiveFlow;
    UINT PacketSize;
} ATM_SETTINGS, *PATM_SETTINGS;


#define MAX_RECV_PDUS 63


typedef struct _MACADDRESS{unsigned char Digit[6];}  MACADDRESS;

typedef enum _ADAPTER_TYPE
{
	NDIS_ADAPTER,
	WDM_ADAPTER
}ADAPTER_TYPE;

/* definitions of network interface structure (one per physical interface)	*/
typedef struct _ATM_ADAPTER                     /* info about one net interface	*/
{				     
	ADAPTER_TYPE AdapterType;
	int          AdapterNumber;
	int          DeviceNumber;
	char		 IdentString[MAX_NAME_LENGTH+1];
	char		 DriverDesc [MAX_NAME_LENGTH+1];
	char         PnpInstanceID[MAX_NAME_LENGTH+1];
	ULONG        Speed;
    ULONG        MemorySize;
    ULONG        LinkStatus;
    MACADDRESS	 MacAddress;
    DWORD        DriverVersion;
} ATM_ADAPTER, *PATM_ADAPTER, **PPATM_ADAPTER;



typedef struct _ATM_APPLICATION_INFO
{
    ULONG MemorySize;
    ULONG Speed;
    char  MacAddress[6];
    ULONG LinkStatus;
    ULONG DriverVersion;

} ATM_APPLICATION_INFO, *PATM_APPLICATION_INFO;

#define OID_ATM_APPLICATION_INFO  0x080102F0
#define OID_ATM_SET_LOOPBACK_MODE 0x080102F1


typedef struct _ATM_PVC
{
	BOOL         bOpened;
	HANDLE       hSocket;
	PATM_ADAPTER pAdapter;
	ATM_SETTINGS ATMSettings;
	DWORD        dwUserThreadID;
    DWORD        dwReceiveThreadID;
	HANDLE       hReceiveThreadHandle;
	HANDLE       hReceiveCloseEvent;
	HANDLE       hReceiveMutex;
	HANDLE       hTransmitMutex;
	LIST_HEAD    ReceiveQueue;
}ATM_PVC, *PATM_PVC;

typedef struct _PDU
{
	LIST_HEAD  Link;
	ULONG      uLength;
	CHAR FAR  *pBuffer;
}PDU, *PPDU;

#pragma pack(push, 1)
typedef struct _PROTOCOL_OID_DATA 
{
    ULONG           Oid;
    ULONG           Length;
    unsigned char   Data[1];
}   
PROTOCOL_OID_DATA, *PPROTOCOL_OID_DATA;
#pragma pack(pop)



typedef struct _PROTOCOL_DRIVER_REQUEST 
{
  HANDLE       hEvent;
  OVERLAPPED   OverLapped;
  PVOID        Buffer;
  UINT         Length;
  ULONG		   ulBytesReceived;
  int		   bIoComplete;
} PROTOCOL_DRIVER_REQUEST, *LPPROTOCOL_DRIVER_REQUEST;

typedef struct _TYPE_ENTRY
{
	BOOL (*GetATMAdapters)( PATM_ADAPTER, DWORD *);
	BOOL (*GetATMInformation) ( PATM_ADAPTER);
	BOOL (*CreateATMPVC)( PATM_PVC);
	BOOL (*DeleteATMPVC)( PATM_PVC);
	int (*WriteATMPVC)(PATM_PVC, char *, int);

} TYPE_ENTRY, *PTYPE_ENTRY; 


LPPROTOCOL_DRIVER_REQUEST PROATMAllocateRequestData( int Length );
VOID  PROATMFreeRequestData( LPPROTOCOL_DRIVER_REQUEST pPROATMRequest );
int   PROATMDeviceIoControl( HANDLE handle, LPPROTOCOL_DRIVER_REQUEST pPROATMRequest,  ULONG ulIoctl,  int bSync );

int  proatmInitAdapters(void);
void proatmExitAdapters(void);
int  proatmOpenVC(int adapterNumber, int vpi, int vci);
int  proatmCloseVC(int  vcNumber);
int  proatmReadPDU(int  vcNumber, char *pPDU, int pduSize);
int  proatmWritePDU(int vcNumber, char *pPDU, int pduSize);

#endif //_PROATMAPI_H_

